[<p align="right">->Back to directory</p>](0.directory.md)  

Considering the cost of the connection from the very beginning of the design, the easiest way.
## TCP
* Any client that supports TCP can link to the server.
* Note that TCP connections are not open for password security, so you should decide whether to block the TCP port of the server or disable the TCP service directly in the configuration file.

Simple debugging can use tools such as telnet, or you can write your own client program. For the specific communication interaction protocol, please refer to the chapter "Protocol" and "Command" in the document.      

Let's take a simple Go TCP Client Demo:      
~~~shell
package main

import (
	"bufio"
	"fmt"
	"log"
	"net"
	"os"
	"time"
)

func main() {
	//5S Timeout
	connTimeout := 5 * time.Second
	//connect to the server
	conn, err := net.DialTimeout("tcp", "127.0.0.1:7720", connTimeout)
	Check_Error("dial failed:", err, true) //Error record
	defer conn.Close()

	//Settings with the server read and write response timeout
	readTimeout := 7 * time.Second
	err = conn.SetReadDeadline(time.Now().Add(readTimeout)) // timeout
	Check_Error("setReadDeadline failed:", err, false)      //Error record

	//Read the server's welcome data first. "Owlcache TCP Link Success..."
	hello_message, hello_err := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read the welcome information error:", hello_err, false) //Error record
	fmt.Print("Message from server: " + hello_message)

	//Send the command statement that needs to be queried
	fmt.Fprintf(conn, "get hello\n")
	message1, cmd_err1 := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read Owlcache Data Error:", cmd_err1, false) //Error record
	fmt.Print("Message from server: " + message1)

	//Send the command statement that needs to be queried
	fmt.Fprintf(conn, "get hello info\n")
	message2, cmd_err2 := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read Owlcache Data Error:", cmd_err2, false) //Error record
	fmt.Print("Message from server: " + message2)

}

//Error record
func Check_Error(title string, err error, is_exit bool) {

	if err != nil {
		log.Println(title, err)
		if is_exit {
			os.Exit(0)
		}
	}

}

~~~
The result of the response is as follows:    
~~~shell
Message from server: Owlcache TCP Link Success...
Message from server: world
Message from server: {"Cmd":"get","Status":200,"Results":"SUCCESS","Key":"hello","Data":"","ResponseHost":"127.0.0.1:7720","KeyCreateTime":"2021-12-07T09:36:39.1568835+08:00"}
~~~


## HTTP
* Any client that supports HTTP can connect to the server.
* The way HTTP connections are in the main direction of owlcache is also recommended for this connection.

Simple debugging can use browsers, CURL and other tools, or you can write your own HTTP client program. For the specific communication interaction protocol, please refer to the chapter "Communication Protocol" and "Operation Command" in the document.