[<p align="right">->返回目录</p>](0.directory.md)  

考虑到连接成本从最开始的设计就采用最简单的方式。
## TCP
* 任意支持TCP的客户端都可以链接服务端。
* 注意TCP连接出于安全考虑，不设置密码开放访问，所以您在线上实施时要按需求来决定是否要屏蔽服务器TCP的端口，或者在配置文件中直接关闭TCP服务。

简单调试可以使用telnet等工具，也可以自己写客户端程序。具体通讯交互协议请参考文档中的“通讯协议”“操作命令”章节。  

下面提供一个简单Go TCP Client Demo:      
~~~shell
package main

import (
	"bufio"
	"fmt"
	"log"
	"net"
	"os"
	"time"
)

func main() {
	//5S Timeout
	connTimeout := 5 * time.Second
	//连接服务器
	conn, err := net.DialTimeout("tcp", "127.0.0.1:7720", connTimeout)
	Check_Error("dial failed:", err, true) //错误记录
	defer conn.Close()

	//设置与服务器读写响应超时
	readTimeout := 7 * time.Second
	err = conn.SetReadDeadline(time.Now().Add(readTimeout)) // timeout
	Check_Error("setReadDeadline failed:", err, false)      //错误记录

	//先读取服务器的欢迎数据 "Owlcache TCP Link Success..."
	hello_message, hello_err := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read the welcome information error:", hello_err, false) //错误记录
	fmt.Print("Message from server: " + hello_message)

	//向服务器发送需要查询的命令语句
	fmt.Fprintf(conn, "get hello\n")
	message1, cmd_err1 := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read Owlcache Data Error:", cmd_err1, false) //错误记录
	fmt.Print("Message from server: " + message1)

	//向服务器发送需要查询的命令语句
	fmt.Fprintf(conn, "get hello info\n")
	message2, cmd_err2 := bufio.NewReader(conn).ReadString('\n')
	Check_Error("Read Owlcache Data Error:", cmd_err2, false) //错误记录
	fmt.Print("Message from server: " + message2)

}

//简单封装，简化语句
func Check_Error(title string, err error, is_exit bool) {

	if err != nil {
		log.Println(title, err)
		if is_exit {
			os.Exit(0)
		}
	}

}

~~~
响应结果如下:    
~~~shell
Message from server: Owlcache TCP Link Success...
Message from server: world
Message from server: {"Cmd":"get","Status":200,"Results":"SUCCESS","Key":"hello","Data":"","ResponseHost":"127.0.0.1:7720","KeyCreateTime":"2021-12-07T09:36:39.1568835+08:00"}
~~~


## HTTP
* 任意支持HTTP的客户端都可以连接服务端。
* HTTP连接的方式为owlcache的主要方向，也推荐这种方式连接。

简单调试可以使用浏览器、CURL等工具，也可以自己写HTTP客户端程序。具体通讯交互协议请参考文档中的“通讯协议”“操作命令”章节。